Class: Takagi::Router::MetadataExtractor::MetadataExtractionRequest

Inherits:
Object
  • Object
show all
Defined in:
lib/takagi/router/metadata_extractor.rb

Overview

Mock request object used during metadata extraction Provides minimal interface to prevent errors when handlers are executed at boot time

Instance Method Summary collapse

Dynamic Method Handling

This class handles dynamic methods through the method_missing method

#method_missing(_name, *_args, &_block) ⇒ Object



123
124
125
# File 'lib/takagi/router/metadata_extractor.rb', line 123

def method_missing(_name, *_args, &_block)
  nil # Return nil for any method calls to prevent errors
end

Instance Method Details

#respond_to_missing?(_name, _include_private = false) ⇒ Boolean

Returns:

  • (Boolean)


127
128
129
# File 'lib/takagi/router/metadata_extractor.rb', line 127

def respond_to_missing?(_name, _include_private = false)
  true # Pretend to respond to everything
end

#to_response(*_args) ⇒ Object



119
120
121
# File 'lib/takagi/router/metadata_extractor.rb', line 119

def to_response(*_args)
  nil # Ignore response generation during metadata extraction
end